/*
Module : PushPin.h
Purpose: Interface of a push pin MFC button class
Created: PJN / 04-04-1996

Copyright (c) 1997 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __PUSHPIN_H__
#define __PUSHPIN_H__

#ifndef CPUSHPINBUTTON_EXT_CLASS
#define CPUSHPINBUTTON_EXT_CLASS
#endif //#ifndef CPUSHPINBUTTON_EXT_CLASS


//////////////////// Classes //////////////////////////////////////////////////

class CPUSHPINBUTTON_EXT_CLASS CPushPinButton : public CButton
{
public:
//Constructors / Destructors
  CPushPinButton() noexcept;
  CPushPinButton(const CPushPinButton&) = delete;
  CPushPinButton(CPushPinButton&&) = delete;
  ~CPushPinButton() = default; //NOLINT(modernize-use-override)

//General Methods
  CPushPinButton& operator=(const CPushPinButton&) = delete;
  CPushPinButton& operator=(CPushPinButton&&) = delete;
  BOOL Create(_In_ const RECT& rect, _In_opt_ CWnd* pParentWnd, _In_ UINT nID); //NOLINT(clang-diagnostic-overloaded-virtual)
  BOOL SetBitmapIDs(_In_ UINT uiCombnBmpID, _In_ bool b6Buttons = true);

//Pinning support
  void SetPinned(_In_ bool bPinned);
  _NODISCARD bool IsPinned() const noexcept;

//Should be called in response to system color changes
  bool ReloadBitmaps();

protected:
//Enums
  enum class ButtonState
  {
    UNPINNED_NORMAL = 0,
    UNPINNED_FLYBY = 1,
    UNPINNED_DOWN = 2,
    PINNED_NORMAL = 3,
    PINNED_DOWN = 4,
    PINNED_FLYBY = 5,
    MAX_BUTTON_INDEX = 6,
  };
  
//Methods
  void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) override;
  void PreSubclassWindow() override;
  bool SizeToContent();
  bool LoadBitmaps();

//Message handlers
  afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
  afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
  afx_msg void OnMouseMove(UINT nFlags, CPoint point);
  afx_msg void OnCaptureChanged(CWnd* pWnd);
  afx_msg LPARAM OnMouseLeave(WPARAM wParam, LPARAM lParam);

//Member variables
  CRect m_MaxRect;
  bool m_bCaptured;
  ButtonState m_buttonState;
  bool m_bTrackLeave;
  UINT m_uiCombnBmpID;
  int m_nButtons;
  CBitmap m_bmpCombined;

  DECLARE_MESSAGE_MAP()
};

#endif //#ifndef __PUSHPIN_H__
